Attribute VB_Name = "bFolder"
Option Explicit

'###########################################
'
'                     MODUL "FolderBrowse"
'
'                    (c) Ingo Steinhaus 2000
'                    ingo.steinhaus@gmx.de
'
'  Funktionen zur Anzeige des Windows-Standardialogs zur Auswahl
'  eines Ordners auf dem aktiven Rechner oder im Netzwerk.
'
'  Dieses Modul ist urheberrechtlich geschtzte Freeware.
'  Die originale Copyright-Meldung darf nicht entfernt oder ver-
'  ndert werden. Der Quelltext darf nicht verndert werden.
'
'###########################################

Private Const MAX_PATH = 260

'*********************************************************
' Die Datenstruktur "BrowseInfo" dient der Konfiguration
' des Folder-Browse-Dialogs.

Private Type BrowseInfo
  hWndOwner As Long
  'Handle des Besitzers (mit GetActiveWindow() abfragen)
  pIDLRoot As Long
  'Adresse der IID-Liste.
  'Sie gibt die Position des Wurzelordners an, der als
  'Baumwurzel im Browse-Dialog erscheint. Nur dieser
  'Ordner und die davon abzweigenden Ordner erscheinen
  'im Browse-Dialog.
  'Sie knnen hier NULL eintragen; in diesem Fall wird
  'der Desktop als Baumwurzel benutzt. Dadurch haben Sie
  'Zugriff auf alle Laufwerke sowie die Netzwerkumgebung.
  pszDisplayName As Long
  'Adresse eines Puffers, der den Namen des vom Anwender
  'ausgewhlten Ordners (ohne Pfad) enthlt. Der Puffer
  'kann maximal 260 Zeichen enthalten (MAX_PATH Konstante)
  lpszTitle As Long
  'Adresse eines nullterminierten Strings, der ber der
  'Baumansicht gezeigt werden. Sie knnen diesen String
  'fr Informationen oder Anweisungen benutzen.
  ulFlags As Long
  'Flags, die die Anzeigeoptionen des Dialogfeldes bestimmen
  lpfnCallback As Long
  'Adresse einer Rckruffunktion, die in der Anwendung
  'definiert wird.
  'Sie knnen hier NULL eintragen.
  lParam As Long
  'Ein anwendungsdefinierter Wert, den das Dialogfeld an
  'eine Rckruffunktion bergibt.
  iImage As Long
  'Eine Variable fr das Bild, mit dem der ausgewhlte Ordner
  'in der Baumansicht gekennzeichnet ist. Die Variable enthlt
  'einen Index auf die Systembilderliste von Windows 95/98.
End Type

'*********************************************************
' Die folgenden Konstanten sind die erlaubten Werte fr
' BrowseInfo->ulFlags.

Private Const BIF_BROWSEFORCOMPUTER = &H1000
'Nur Computer als Auswahl erlaubt. Wenn der Anwender andere
'Ordner markiert, kann der OK-Schalter nicht ausgewhlt
'werden.

Private Const BIF_BROWSEFORPRINTER = &H2000
'Nur Drucker als Auswahl erlaubt. Wenn der Anwender andere
'Ordner markiert, kann der OK-Schalter nicht ausgewhlt
'werden.

Private Const BIF_BROWSEINCLUDEFILES = &H4000
'Der Dialog zeigt neben den Ordnern auch Dateien.

Private Const BIF_DONTGOBELOWDOMAIN = &H2
'Der Dialog zeigt keine Netzwerkordner unterhalb der
'aktuellen Domain.

Private Const BIF_RETURNFSANCESTORS = &H8
'Nur Dateisystemobjekte als Auswahl erlaubt. Wenn der
'Anwender andere Ordner markiert, kann der OK-Schalter
'nicht ausgewhlt werden.

Private Const BIF_RETURNONLYFSDIRS = &H1
'Nur Dateisystemordner als Auswahl erlaubt. Wenn der
'Anwender andere Ordner markiert, kann der OK-Schalter
'nicht ausgewhlt werden.

Private Const BIF_STATUSTEXT = &H4
'Der Dialog enthlt eine Statuszeile. Die Rckruffunktion
'kann die Statuszeile ausfllen.

'*********************************************************
'*** CoTaskMemFree ***
' Eine Funktion zum Verwerfen von angefordertem globalen
' Speicher.

Private Declare Sub CoTaskMemFree Lib "ole32.dll" _
  (ByVal hMem As Long)

'*********************************************************
'*** lstrcat ***
' Eine Funktion zum Verknpfen von nullterminierten Strings.

Private Declare Function lstrcat Lib "kernel32" Alias _
  "lstrcatA" (ByVal lpString1 As String, _
  ByVal lpString2 As String) As Long

'*********************************************************
' *** GetActiveWindow ***
' Eine Funktion zum Ermitteln des Fenster-Handles.

Private Declare Function GetActiveWindow Lib "user32" () _
  As Long

'*********************************************************
' *** SHGetPathFromIDList ***
' Diese Funktion konvertiert eine IID-Liste in einen Pfad des
' Dateisystems.

Private Declare Function SHGetPathFromIDList Lib "shell32" _
  (ByVal pidList As Long, ByVal lpBuffer As String) As Long

'********************************************************
Rem *** SHBrowseForFolder ***
Rem Diese Funktion ruft den Folder-Browse-Dialog auf.
Rem Der Aufrufer mu den Speicher der IID-Liste verwerfen.

Private Declare Function SHBrowseForFolder Lib "shell32" _
  (lpbi As BrowseInfo) As Long

'*********************************************************
' *** BrowseForFolder ***
' Eine VB/VBA-Funktion als einfach zu nutzender Mantel fr den
' Aufruf des Folder-Browse-Dialogs.

Public Function BrowseForFolder(Prompt As String) As String
  
  Dim n As Integer
  Dim IDList As Long
  Dim Result As Long
  Dim ThePath As String
  Dim BI As BrowseInfo

  'Erzeugen der Datenstruktur
  With BI
    'Handle des aktiven Fensters ermitteln
    .hWndOwner = GetActiveWindow()
    'Titel des Dialoges
    .lpszTitle = lstrcat(Prompt, "")
    'Nur Dateisystemordner erlaubt
    .ulFlags = BIF_RETURNONLYFSDIRS
  End With

  'Anzeigen des Dialogs und bergabe an eine IID-Liste
  IDList = SHBrowseForFolder(BI)

  'Wenn IDList > 0, dann Auswahl bearbeiten
  If IDList Then
    'Speicher anfordern
    ThePath = String$(MAX_PATH, 0)
    'IID-Liste in Pfadangabe konvertieren
    Result = SHGetPathFromIDList(IDList, ThePath)
    'Speicher fr IID-Liste verwerfen
    Call CoTaskMemFree(IDList)
    'Alle Bytes hinter Nullbyte verwerfen
    n = InStr(ThePath, vbNullChar)
    If n Then ThePath = Left$(ThePath, n - 1)
  End If

  'Rckgabewert der Funktion definieren
  BrowseForFolder = ThePath
End Function


